## Stand Alone Script for the B2B observation imaging                      
## (Customized for uid://A002/Xf07268/X32f0, R Lep Band9 B2B)           
## Script to image the B2B B4 to B9
## R Lep testing data 
## NOTE this is not an SV dataset

## One round of selfcal is made using the continuum
## there are no lines detected in initial analysis
## and no image cubes are made. The Primary beam
## corrections are not applied. 

## Below 'EXTRA NOTES' sections provide some 
## more context of the steps 

### Hardcoded script Final Version
### Yoshiharu Asaki JAO & Luke Maud ESO
### For further information please contact:
### yasaki@alma.cl   JAO/NAOJ
### lmaud@eso.org   ESO
###
### If using these data please reference:
### Asaki, Y., Maud, L. T., et al. 2023
### and B2B papers therein and published thereafter

### 27/02/2023 - CASA 6.4.1-12 version

### IMPORTANT
### SPWs 33,35,37,39,41,43,45,47 are the BAND 9 - science target frequency
### SPWs 64,66 are the BAND 4 - reference calibrator frequency
### these are in the image names 

thesteps = []
step_title = {0: 'Image the calibrators',
              1: 'Image Continuum R Lep',
              2: 'Selfcal and image R Lep',
              3: 'Copy out fits images'
              # end of file - commands to reset flags
             }

if 'applyonly' not in globals(): applyonly = False
try:
  print('List of steps to be executed ...'+str(mysteps))
  thesteps = mysteps
except:
  print('global variable mysteps not set.')
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print('Executing all steps: ', thesteps)

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]
# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

import casalith

import glob

if re.search('^6.4.1', '.'.join([str(i) for i in casalith.version()[:-1]])) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 6.4.1')

mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  ## EXTRA NOTE - mask for point source calibrators is fixed as 
  ##              a central circle. Image parameters are also
  ##              provided to make a smaller focused image
  ##              to reduce imaging time

  cell_size_im = '0.001arcsec'
  image_size = 512
  centre_mask = ['circle[[256pix,256pix],10pix]']
  nits = 100

  # Imaging the Bandpass - only HF
  # EXTRA NOTE - the BP and DGC are the same
  #              here image only the BP scan

  os.system('rm -rf J0522-3627_bp.spw33_35_37_39_41_43_45_47.cont*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='J0522-3627_bp.spw33_35_37_39_41_43_45_47.cont', 
         field='J0522-3627',
         interactive=False,
         spw = '33,35,37,39,41,43,45,47',
         scan='3',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


  # DGC at HF - B9 
  # EXTRA NOTE - Image of DGC use DGC scans only
  os.system('rm -rf J0522-3627_dgc.spw33_35_37_39_41_43_45_47.cont*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='J0522-3627_dgc.spw33_35_37_39_41_43_45_47.cont', 
         field='J0522-3627',
         scan='11,13,15,17,19,81,83,85,87,89,153,155,157,159,161,227,229,231,233,235',
         interactive=False,
         spw = '33,35,37,39,41,43,45,47',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


  # FLUX cal - only HF
  # EXTRA NOTE - recall this is not actually used
  #             for flux scaling in calibration
  #             and itself is amplitude calibrated by
  #             the DGC/BP source J0522-3627
  os.system('rm -rf J0423-0120_flux.spw33_35_37_39_41_43_45_47.cont*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='J0423-0120_flux.spw33_35_37_39_41_43_45_47.cont', 
         field='J0423-0120',
         interactive=False,
         spw = '33,35,37,39,41,43,45,47',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


  # CHECK source at HF - B9
  # EXTRA NOTE - The check source is also located 'further'
  #              from the phase cal (~7deg) than the target
  #              thus it is not expected that Phase transfer is
  #              perfect and the check source will not appear
  #              very point like. See also Maud et al 2023 

  os.system('rm -rf J0438-1251_chk.spw33_35_37_39_41_43_45_47.cont*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='J0438-1251_chk.spw33_35_37_39_41_43_45_47.cont', 
         field='J0438-1251',
         interactive=False,
         spw = '33,35,37,39,41,43,45,47',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


  # DGC at LF - B4 
  # EXTRA NOTE - these are LF DGC scans only 
  os.system('rm -rf J0522-3627_dgc.spw64_66.cont*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='J0522-3627_dgc.spw64_66.cont', 
         field='J0522-3627',
         scan='10,12,14,16,18,20,80,82,84,86,88,90,152,154,156,158,160,162,226,228,230,232,234,236',
         interactive=False,
         spw = '64,66',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)


  # PHASE cal at LF - B4

  os.system('rm -rf J0504-1512_ph.spw64_66.cont*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='J0504-1512_ph.spw64_66.cont', 
         field='J0504-1512',
         interactive=False,
         spw = '64,66',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs',
         robust=0.5,
         mask=centre_mask)




mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])


  # We save the flags before any selfcal or imaging
  # so it is a state we can return to
  flagmanager(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
      mode = 'save',
      versionname = 'beforeSelfCal')

  # in case or any rerun remove the model
  delmod(vis= 'uid___A002_Xf07268_X32f0.ms.split.cal', field='R_Lep') 

  cell_size_im = '0.001arcsec'
  image_size = 512
  nits = 1000

  # Target continuum
  os.system('rm -rf R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual.*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual',
         field='R_Lep',
         datacolumn = 'data',
         interactive=False,
         spw='33,35,37,39,41,43,45,47',         
         deconvolver='hogbom',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs', 
         robust=0.5,
         usemask='auto-multithresh',
         sidelobethreshold=2.0,
         noisethreshold=3.00,
         lownoisethreshold=1.0,
         minbeamfrac=0.3,
         growiterations=75,
         negativethreshold=0.0,
         savemodel='modelcolumn')




# Target self-cal and imaging
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  cell_size_im = '0.001arcsec'
  image_size = 512
  nits = 1000


  ## EXTRA NOTES - we here try only a INF scan based
  ##               selfcalibration on the continuum
  ##               as R Lep is relatively weak there
  ##               are some flags and this looses the
  ##               longest baselines
  os.system('rm -rf R_Lep.p1*')
  gaincal(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         caltable='R_Lep.p1',
         solint='inf',
         calmode='p',
         gaintype='T',  # was G cnow comb pols 
         spw = '33,35,37,39,41,43,45,47',
         combine = 'spw',
         minsnr=3.0,
         refant='DV02',
         field='R_Lep')




  ## EXTRA NOTE - once the selfcal is applied any flags that were
  ##              created from the gaincal step are permanatly 
  ##              appended to the MS, above (step 1) saved the flag state
  ##              so it can be returned to 
  applycal(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         field = 'R_Lep',
         spw = '33,35,37,39,41,43,45,47',
         spwmap = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,33,36,33,38,33,40,33,42,33,44,33,46,33],
         interp='linearPD',
         gaintable = 'R_Lep.p1',
         gainfield = '')

  os.system('rm -rf R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-selfcal*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-selfcal',
         field='R_Lep',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         datacolumn= 'corrected',
         interactive=False,
         spw='33,35,37,39,41,43,45,47',         
         deconvolver='hogbom',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs', 
         robust=0.5,
         usemask='auto-multithresh',
         sidelobethreshold=2.0,
         noisethreshold=3.00,
         lownoisethreshold=1.0,
         minbeamfrac=0.3,
         growiterations=75,
         negativethreshold=0.0,
         savemodel='modelcolumn')



  ## EXTRA NOTE - here we ALSO image R Lep but using the 
  ##              data column. The main point of this is
  ##              that self-calibration flagged a lot of 
  ##              the longer baselines, so we want to make
  ##              and image of the original data, with the 
  ##              same flags as applied by selfcal

  os.system('rm -rf R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-dataselfcalflags*')
  tclean(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
         imagename='R_Lep_sci.spw33_35_37_39_41_43_45_47.cont.I.manual-dataselfcalflags',
         field='R_Lep',
         phasecenter='ICRS 04h59m36.359000s -14d48m22.531000s',
         datacolumn= 'data', ## DATA column, i.e. phase referencing only 
         interactive=False,
         spw='33,35,37,39,41,43,45,47',         
         deconvolver='hogbom',
         cell=cell_size_im, imsize=image_size, niter=nits,
         specmode='cont',
         weighting='briggs', 
         robust=0.5,
         usemask='auto-multithresh',
         sidelobethreshold=2.0,
         noisethreshold=3.00,
         lownoisethreshold=1.0,
         minbeamfrac=0.3,
         growiterations=75,
         negativethreshold=0.0)

mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print('\nStep '+str(mystep)+' '+step_title[mystep])

  imagetofits=glob.glob('*.image*') 

  for imagefit in imagetofits: 
      exportfits(imagefit, fitsimage=imagefit+'.fits') 



## extra command for restoring flags that continuum selfcal made
## BE AWARE - you should not image the corrected column (after selfcal)
## with the origonal flag state (pre selfcal) as the longer baselines
## are now without any solutions other than as phase referenced and
## this is not a good practice, i.e. short baseline with some selfcal
## long baseline with low SNR and no selfcal but still included

#  flagmanager(vis = 'uid___A002_Xf07268_X32f0.ms.split.cal',
#      mode = 'restore',
#      versionname = 'beforeSelfCal')
